@@ -1,4 +1,5 @@
 #### For both 'Extract' ("[") and 'Replace' ("[<-") Method testing
+####    aka    subsetting     and  subassignment
 
 #### suppressPackageStartupMessages(...)  as we have an *.Rout.save to Rdiff against
 stopifnot(suppressPackageStartupMessages(require(Matrix)))
@@ -68,7 +69,7 @@ stopifnot(identical(mn["rc", "D"], mn[3,4]), mn[3,4] == 24,
 	  identical(unname(z), zz),
 	  identical(a.m, array(v, dim=dim(mn), dimnames=dimnames(mn)))
 	  )
-
+showProc.time()
 
 ## Printing sparse colnames:
 ms[sample(28, 20)] <- 0
@@ -108,58 +109,111 @@ stopifnot(is(sel, "lMatrix"), is(ssel, "lsparseMatrix"),
 	  identical3(m[ sel],  m[ ssel], as.matrix(m)[as.matrix( ssel)]),
 	  identical3(m[!sel],  m[!ssel], as.matrix(m)[as.matrix(!ssel)])
 	  )
+showProc.time()
 
 ## more sparse Matrices --------------------------------------
 
+##' @title Check sparseMatrix sub-assignment   m[i,j] <- v
+##' @param ms sparse Matrix
+##' @param mm its [traditional matrix]-equivalent
+##' @param k  (approximate) length of index vectors (i,j)
+##' @param n.uniq (approximate) number of unique values in  i,j
+##' @param show logical; if TRUE, it will not stop on error
+##' @return
+##' @author Martin Maechler
+chkAssign <- function(ms, mm = as(ms, "matrix"),
+                      k = min(20,dim(mm)), n.uniq = k %/% 3,
+                      vRNG = { if(is.numeric(mm) || is.complex(mm))
+                                   function(n) rpois(n,lambda= 0.75)# <- about 47% zeros
+                      else ## logical
+                          function(n) runif(n) > 0.8 }, ## 80% zeros
+                      show=FALSE)
+{
+    stopifnot(is(ms,"sparseMatrix"))
+    s1 <- function(n) sample(n, pmin(n, pmax(1, rpois(1, n.uniq))))
+    i <- sample(s1(nrow(ms)), k/2+ rpois(1, k/2), replace = TRUE)
+    j <- sample(s1(ncol(ms)), k/2+ rpois(1, k/2), replace = TRUE)
+    assert.EQ.mat(ms[i,j], mm[i,j])
+    ## now sub*assign* to these repeated indices, and then compare -----
+    v <- vRNG(length(i) * length(j))
+    mm[i,j] <- v
+    ms[i,j] <- v
+    if(!show) { op <- options(error = recover); on.exit(options(op)) }
+    assert.EQ.mat(ms, mm, show=show)
+}
+
+## Get duplicated index {because these are "hard" (and rare)
+getDuplIndex <- function(n, k) {
+    repeat {
+        i <- sample(n, k, replace=TRUE) # 3 4 6 9 2 9 :  9 is twice
+        if(anyDuplicated(i)) break
+    }
+    i
+}
+
+## From package 'sfsmisc':
+repChar <- function (char, no) paste(rep.int(char, no), collapse = "")
+
 m <- 1:800
 set.seed(101) ; m[sample(800, 600)] <- 0
 m0 <- Matrix(m, nrow = 40)
 m1 <- add.simpleDimnames(m0)
-for(m in list(m0,m1)) { ## -- with and without dimnames
-mm <- as(m, "matrix")
-str(mC <- as(m, "dgCMatrix"))
-str(mT <- as(m, "dgTMatrix"))
-stopifnot(identical(mT, as(mC, "dgTMatrix")),
-	  identical(mC, as(mT, "dgCMatrix")),
-	  Qidentical(mC[0,0], new("dgCMatrix")),
-	  Qidentical(mT[0,0], new("dgTMatrix")),
-	  identical(unname(mT[0,]), new("dgTMatrix", Dim = c(0L,20L))),
-	  identical(unname(mT[,0]), new("dgTMatrix", Dim = c(40L,0L))),
-	  identical(mC[0,], as(mT[FALSE,], "dgCMatrix")),
-	  identical(mC[,0], as(mT[,FALSE], "dgCMatrix")),
-	  sapply(c(0:2, 5:10), function(k) {i <- seq_len(k); all(mC[i,i] == mT[i,i])}),
-	  TRUE)
-cat("ok\n")
-show(mC[,1])
-show(mC[1:2,])
-show(mC[7,  drop = FALSE])
-assert.EQ.mat(mC[1:2,], mm[1:2,])
-assert.EQ.mat(mC[0,], mm[0,])
-assert.EQ.mat(mC[,FALSE], mm[,FALSE])
-##
-## *repeated* (aka 'duplicated') indices - did not work at all ...
-i <- rep(8:10,2)
-j <- c(2:4, 4:3)
-assert.EQ.mat(mC[i,], mm[i,])
-assert.EQ.mat(mC[,j], mm[,j])
-## FIXME? assert.EQ.mat(mC[,NA], mm[,NA]) -- mC[,NA] is all 0 "instead" of all NA
-## MM currently thinks we should  NOT  allow  <sparse>[ <NA> ]
-assert.EQ.mat(mC[i, 2:1], mm[i, 2:1])
-assert.EQ.mat(mC[c(4,1,2:1), j], mm[c(4,1,2:1), j])
-assert.EQ.mat(mC[i,j], mm[i,j])
-set.seed(7)
-cat(" for(): ")
-for(n in 1:50) {
-    i <- sample(sample(nrow(mC), 7), 20, replace = TRUE)
-    j <- sample(sample(ncol(mC), 6), 17, replace = TRUE)
+for(kind in c("n", "l", "d")) {
+ for(m in list(m0,m1)) { ## -- with and without dimnames -------------------------
+    kClass <- paste(kind, "Matrix", sep="")
+    Ckind <- paste(kind, "gCMatrix", sep="")
+    Tkind <- paste(kind, "gTMatrix", sep="")
+    str(mC <- as(m, Ckind))
+    str(mT <- as(as(as(m, kClass), "TsparseMatrix"), Tkind))
+    mm <- as(mC, "matrix") # also logical or double
+    IDENT <- if(kind == "n") function(x,y) Q.eq2(x,y, tol=0) else identical
+    stopifnot(identical(mT, as(as(mC, "TsparseMatrix"), Tkind)),
+              identical(mC, as(mT, Ckind)),
+              Qidentical(mC[0,0], new(Ckind)),
+              Qidentical(mT[0,0], new(Tkind)),
+              identical(unname(mT[0,]), new(Tkind, Dim = c(0L,20L))),
+              identical(unname(mT[,0]), new(Tkind, Dim = c(40L,0L))),
+              IDENT(mC[0,], as(mT[FALSE,], Ckind)),
+              IDENT(mC[,0], as(mT[,FALSE], Ckind)),
+              sapply(c(0:2, 5:10),
+                     function(k) {i <- seq_len(k); all(mC[i,i] == mT[i,i])}),
+              TRUE)
+    cat("ok\n")
+    show(mC[,1])
+    show(mC[1:2,])
+    show(mC[7,  drop = FALSE])
+    assert.EQ.mat(mC[1:2,], mm[1:2,])
+    assert.EQ.mat(mC[0,], mm[0,])
+    assert.EQ.mat(mC[,FALSE], mm[,FALSE])
+    ##
+    ## *repeated* (aka 'duplicated') indices - did not work at all ...
+    i <- rep(8:10,2)
+    j <- c(2:4, 4:3)
+    assert.EQ.mat(mC[i,], mm[i,])
+    assert.EQ.mat(mC[,j], mm[,j])
+    ## FIXME? assert.EQ.mat(mC[,NA], mm[,NA]) -- mC[,NA] is all 0 "instead" of all NA
+    ## MM currently thinks we should  NOT  allow  <sparse>[ <NA> ]
+    assert.EQ.mat(mC[i, 2:1], mm[i, 2:1])
+    assert.EQ.mat(mC[c(4,1,2:1), j], mm[c(4,1,2:1), j])
     assert.EQ.mat(mC[i,j], mm[i,j])
-}
-cat("ok\n----\n")
-}## end{for}
+    ##
+    ## set.seed(7)
+    cat(" for(): ")
+    for(n in 1:50) {
+        chkAssign(mC, mm)
+        chkAssign(mC[-3,-2], mm[-3,-2])
+        cat(".")
+    }
+    cat(sprintf("\n[Ok]%s\n\n", repChar("-", 64)))
+ }
+ cat(sprintf("\nok( %s )\n== ###%s\n\n", kind, repChar("=", 70)))
+}## end{for}---------------------------------------------------------------
+showProc.time()
 
 ##---- Symmetric indexing of symmetric Matrix ----------
-m. <- mC; m.[, c(2, 7:12)] <- 0
-validObject(S <- crossprod(add.simpleDimnames(m.) %% 100))
+m. <- mC
+m.[, c(2, 7:12)] <- 0
+isValid(S <- crossprod(add.simpleDimnames(m.) %% 100), "dsCMatrix")
 ss <- as(S, "matrix")
 ds <- as(S, "denseMatrix")
 ## NA-indexing of *dense* Matrices: should work as traditionally
@@ -224,7 +278,7 @@ for(n in 1:100) {
     assert.EQ.mat(A.ii, a.[i,i])
     assert.EQ.mat(T[i,i], ss[i,i])
 }
-
+showProc.time()
 
 stopifnot(all.equal(mC[,3], mm[,3]),
 	  identical(mC[ij], mC[ij + 0.4]),
@@ -278,7 +332,10 @@ stopifnot(class(l10) == "lsCMatrix", # symmetric indexing -> symmetric !
 A <- Matrix(0,4,3) ; A[c(1,2,1), 2] <- 1 ; A
 B <- A;              B[c(1,2,1), 2] <- 1:3; B; B. <- B
 B.[3,] <- rbind(4:2)
+## change the diagonal and the upper and lower subdiagonal :
 diag(B.) <- 10 * diag(B.)
+diag(B.[,-1]) <- 5* diag(B.[,-1])
+diag(B.[-1,]) <- 4* diag(B.[-1,]) ; B.
 C <- B.; C[,2] <- C[,2];  C[1,] <- C[1,]; C[2:3,2:1] <- C[2:3,2:1]
 stopifnot(identical(unname(as.matrix(A)),
 		    local({a <- matrix(0,4,3); a[c(1,2,1), 2] <-  1 ; a})),
@@ -295,8 +352,7 @@ stopifnot(identical(C, as(T, "CsparseMatrix")))
 
 ## used to fail
 n <- 5 ## or much larger
-sm <- new("dsTMatrix", i=as.integer(1),j=as.integer(1),
-          Dim=as.integer(c(n,n)), x = 1)
+sm <- new("dsTMatrix", i=1L, j=1L, Dim=as.integer(c(n,n)), x = 1)
 (cm <- as(sm, "CsparseMatrix"))
 sm[2,]
 stopifnot(sm[2,] == c(0:1, rep.int(0,ncol(sm)-2)),
@@ -305,12 +361,55 @@ stopifnot(sm[2,] == c(0:1, rep.int(0,ncol(sm)-2)),
 	  all(sm[,-(1:3)] == t(sm[-(1:3),])), # all(<lge.>)
 	  all(sm[,-(1:3)] == 0)
 	  )
+showProc.time()
+
+##---  "nsparse*" sub-assignment :----------
+M <- Matrix(c(1, rep(0,7), 1:4), 3,4)
+N0 <- kronecker(M,M)
+Nn <- as(N0, "nMatrix"); nn <- as(Nn,"matrix")
+(Nn00 <- Nn0 <- Nn); nn00 <- nn0 <- nn
+
+set.seed(1)
+Nn0 <- Nn00; nn0 <- nn00
+for(i in 1:200) {
+    Nn <- Nn0
+    nn <- nn0
+    i. <- getDuplIndex(nrow(N0), 6)
+    j. <- getDuplIndex(ncol(N0), 4)
+    vv <- sample(c(FALSE,TRUE),
+                 length(i.)*length(j.), replace=TRUE)
+    cat(",")
+    Nn[i., j.] <- vv
+    nn[i., j.] <- vv
+    assert.EQ.mat(Nn, nn)
+    if(!all(Nn == nn)) {
+        cat("i=",i,":\n i. <- "); dput(i.)
+        cat("j. <- "); dput(j.)
+        cat("which(vv): "); dput(which(vv))
+        cat("Difference matrix:\n")
+        show(drop0(Nn - nn))
+    }
+    cat("k")
+    ## sub-assign double precision to logical sparseMatrices should error:
+    ## well... warning for now {back compatibility: gave *no* warning, nothing .. !}:
+    assertWarning(Nn[1:2,] <- -pi)
+    assertWarning(Nn[, 5] <- -pi)
+    assertWarning(Nn[2:4, 5:8] <- -pi)
+    ##
+    cat(".")
+    if(i %% 10 == 0) cat("\n")
+    if(i == 100) {
+        Nn0 <- as(Nn0, "CsparseMatrix")
+        cat("Now: class", class(Nn0)," :\n~~~~~~~~~~~~~~~~~\n")
+    }
+}
+showProc.time()
 
 m0 <- Diagonal(5)
 stopifnot(identical(m0[2,], m0[,2]),
 	  identical(m0[,1], c(1,0,0,0,0)))
 ### Diagonal -- Sparse:
-(m1 <- as(m0, "TsparseMatrix"))  # dtTMatrix
+(m1 <- as(m0, "TsparseMatrix")) # dtTMatrix
 (m2 <- as(m0, "CsparseMatrix")) # dtCMatrix
 m1g <- as(m1, "generalMatrix")
 stopifnot(is(m1g, "dgTMatrix"))
@@ -340,7 +439,7 @@ checkMatrix(M)
 M <- m1; M[1:3, 3] <- 0 ;M
 assert.EQ.mat(M, diag(c(1,1, 0, 1,1)), tol=0)
 T <- m1; T[1:3, 3] <- 10; checkMatrix(T)
-stopifnot(isValid(T, "dtTMatrix"), identical(T[,3], c(10,10,10,0,0)))
+stopifnot(is(T, "triangularMatrix"), identical(T[,3], c(10,10,10,0,0)))
 
 M <- m2; M[1,] <- 0 ; M ; assert.EQ.mat(M, diag(c(0,rep(1,4))), tol=0)
 M <- m2; M[,3] <- 3 ; stopifnot(is(M,"sparseMatrix"), M[,3] == 3)
@@ -349,6 +448,7 @@ M <- m2; M[1:3, 3] <- 0 ;M
 assert.EQ.mat(M, diag(c(1,1, 0, 1,1)), tol=0)
 T <- m2; T[1:3, 3] <- 10; checkMatrix(T)
 stopifnot(is(T, "dtCMatrix"), identical(T[,3], c(10,10,10,0,0)))
+showProc.time()
 
 
 ## "Vector indices" -------------------
@@ -424,6 +524,7 @@ stopifnot(identical4(mc[-(1:4), -(2:4)], mC[5, c(1,5:7)],
 
 ## mixing of negative and positive must give error
 assertError(mT[-1:1,])
+showProc.time()
 
 ## Sub *Assignment* ---- now works (partially):
 mt0 <- mt
@@ -485,6 +586,7 @@ for(i in 1:50) {
     validObject(mc) ; assert.EQ.mat(mc, m)
     validObject(mt) ; assert.EQ.mat(mt, m)
 }
+showProc.time()
 options(Matrix.verbose = TRUE)
 
 mc # no longer has non-structural zeros
@@ -586,6 +688,7 @@ s2[2:1,2:1] <- 4:1
 t2[2:1,2:1] <- 4:1
 assert.EQ.mat(t2, m)# ok
 assert.EQ.mat(s2, m)# failed in 0.9975-8
+showProc.time()
 
 
 ## m[cbind(i,j)] <- value: (2-column matrix subassignment):
@@ -682,6 +785,7 @@ stopifnot(identical(Diagonal(x = 1+ 1:n), dLrg),
           identical(Ctrg, as(dLrg,"CsparseMatrix")))
 
 cc <- capture.output(show(dLrg))# show(<diag>) used to error for large n
+showProc.time()
 
 ## Large Matrix indexing / subassignment
 ## ------------------------------------- (from ex. by Imran Rashid)
@@ -719,7 +823,7 @@ stopifnot(is(f, "ngCMatrix"), is(fx, "dgCMatrix"),
 	  identical(as.logical(fx[,6000]), tC),
 	  identical(thCol,  fx[,5762]))
 
-cat('Time elapsed: ', (.pt <- proc.time()),'\n') # "stats"
+showProc.time()
 ##
 cat("checkMatrix() of all: \n---------\n")
 Sys.setlocale("LC_COLLATE", "C")# to keep ls() reproducible
@@ -727,6 +831,6 @@ for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
     cat(nm, "\n")
     checkMatrix(.m, verbose = FALSE)
 }
-cat('Time elapsed: ', proc.time() - .pt,'\n') # "stats"
+showProc.time()
 
 if(!interactive()) warnings()
